@@ -45,23 +45,23 @@ mod tests {
     }
 
     #[test]
-    fn test_file() -> Result<()> {
+    fn test_aidl() -> Result<()> {
         let input = r#"package x.y.z;
             import a.b.c;
             interface MyInterface {}
         "#;
-        assert_parser!(input, rules::aidl::OptFileParser::new());
+        assert_parser!(input, rules::aidl::OptAidlParser::new());
 
         Ok(())
     }
 
     #[test]
-    fn test_file_with_unrecovered_error() -> Result<()> {
+    fn test_aidl_with_unrecovered_error() -> Result<()> {
         use crate::diagnostic::ParseError;
 
         let input = "wrong, wrong and wrong!";
         let lookup = lookup(input);
-        let res = rules::aidl::OptFileParser::new().parse(&lookup, &mut Vec::new(), input);
+        let res = rules::aidl::OptAidlParser::new().parse(&lookup, &mut Vec::new(), input);
 
         assert!(matches!(res, Err(ParseError::InvalidToken { .. })));
 
@@ -69,13 +69,13 @@ mod tests {
     }
 
     #[test]
-    fn test_file_with_recovered_error() -> Result<()> {
+    fn test_aidl_with_recovered_error() -> Result<()> {
         let input = r#"package x.y.z;
                import a.b.c;
                oops_interface MyInterface {}
            "#;
         let mut diagnostics = Vec::new();
-        assert_parser!(input, rules::aidl::OptFileParser::new(), &mut diagnostics);
+        assert_parser!(input, rules::aidl::OptAidlParser::new(), &mut diagnostics);
 
         assert_diagnostics!(diagnostics, @r###"
         [
@@ -299,7 +299,7 @@ mod tests {
           Diagnostic(
             kind: Error,
             range: "...",
-            message: "Invalid enum element - Unrecognized token `=`\nExpected one of \"{\", BOOLEAN or QUOTED_STRING",
+            message: "Invalid enum element - Unrecognized token `=`\nExpected one of \"{\", BOOLEAN, FLOAT or QUOTED_STRING",
             context_message: Some("unrecognized token"),
             hint: None,
             related_infos: [],
@@ -371,21 +371,10 @@ mod tests {
 
     #[test]
     fn test_method_with_invalid_value() -> Result<()> {
-        let input = "TypeName myMethod() = 1234567891234567;";
-        let mut diagnostics = Vec::new();
-        assert_parser!(input, rules::aidl::MethodParser::new(), &mut diagnostics);
-        assert_diagnostics!(diagnostics, @r###"
-        [
-          Diagnostic(
-            kind: Error,
-            range: "...",
-            message: "Invalid method value: number too large to fit in target type",
-            context_message: None,
-            hint: None,
-            related_infos: [],
-          ),
-        ]
-        "###);
+        let input = "TypeName myMethod() = 12.3;";
+        assert!(rules::aidl::MethodParser::new()
+            .parse(&lookup(input), &mut Vec::new(), input)
+            .is_err());
 
         Ok(())
     }
@@ -463,7 +452,7 @@ mod tests {
     }
 
     #[test]
-    fn test_member_with_javadoc() -> Result<(), Box<dyn std::error::Error>> {
+    fn test_member_with_javadoc() -> Result<()> {
         let input = r#"/**
              * Member documentation
              */
@@ -572,6 +561,24 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn test_type_list_non_generic() -> Result<()> {
+        let input = "List";
+        assert_parser!(input, rules::aidl::TypeParser::new());
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_type_list_invalid() -> Result<()> {
+        let input = "List<A, B>";
+        assert!(rules::aidl::ValueParser::new()
+            .parse(&lookup(input), &mut Vec::new(), input)
+            .is_err());
+
+        Ok(())
+    }
+
     #[test]
     fn test_type_map() -> Result<()> {
         let input = "Map<Key,List<V>>";
@@ -580,6 +587,29 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn test_type_map_non_generic() -> Result<()> {
+        let input = "Map";
+        assert_parser!(input, rules::aidl::TypeParser::new());
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_type_map_invalid() -> Result<()> {
+        let input = "Map<A>";
+        assert!(rules::aidl::ValueParser::new()
+            .parse(&lookup(input), &mut Vec::new(), input)
+            .is_err());
+
+        let input = "Map<A,B,C>";
+        assert!(rules::aidl::ValueParser::new()
+            .parse(&lookup(input), &mut Vec::new(), input)
+            .is_err());
+
+        Ok(())
+    }
+
     #[test]
     fn test_value() -> Result<()> {
         // Numbers
@@ -612,6 +642,29 @@ mod tests {
                 .is_err());
         }
 
+        // Empty objects
+        for input in ["{}", "{ }", "{      }"].into_iter() {
+            assert_eq!(
+                rules::aidl::ValueParser::new().parse(&lookup(input), &mut Vec::new(), input)?,
+                "{}"
+            );
+        }
+
+        // Non-empty objects
+        for input in ["{\"hello{<\"}", "{1}", "{1, 2}", "{1, 2, 3, }"].into_iter() {
+            assert_eq!(
+                rules::aidl::ValueParser::new().parse(&lookup(input), &mut Vec::new(), input)?,
+                "{...}"
+            );
+        }
+
+        // Invalid objects
+        for input in ["{\"hello{<\"", "{1sfewf}", "{1, 2, 3,, }"].into_iter() {
+            assert!(rules::aidl::ValueParser::new()
+                .parse(&lookup(input), &mut Vec::new(), input)
+                .is_err());
+        }
+
         Ok(())
     }
 
